library("deSolve")
### BEGIN P MODEL ####################################################################
Pmodel <- function(Time, State, Pars) {
  with(as.list(c(State, Pars)), {
    LQ   <- kl*s^c #leach for FF, where kl is the hidraulic conductivity
    LQ1   <- kl*s1^c #leach for FF, where kl is the hidraulic conductivity
    Ts   <- eta*s  # transpiration FF
    Ts1   <- eta*s1 # transpiration TFF
    fdec <- s*temp/20  # decomposition FF
    fdec1 <- s1*temp/20 # decomposition TFF
    Iw   <- 0.6*PCon*Pmol*U/1000 # P input with uplift,
    AR    <- kam*(Pv + Pv1) # Ants- monkey redistribution of life vegetation biomass 
    TR    <- kta*(Po + Po1) # termite redistribution of dryed organic matter
    AndInput  <- WeAndes*Pmol/1000000
    MaxInAndR <- AndInput/pin #maximum input from white rivers to flooded forest( Gardner 1990 weathering rates Total-LOWLAND)
    MaxInRN <- 0.0074956504524/pin # maximum input from black rivers to flooded forest( Gardner 1990 weathering rates RNB)
    POtterg_in <- IndOtterC* PconFish*daysyear*0.2/1000000 # inputs by otters and human
    kocc <- kc*kcl*Iw/ke/ke
    
    #where PCon is the P concentration of the rock, 
    #aP are the g P/mol and U the uplift rates
    # the two sets of equations are almost the same 
    #with the difference that the flooded forest runs 
    #with a soil moisture of 0.7 the animal inputs from rivers are twice as big
    
    ######### BALANCE FLOODED FOREST #####################################################
    
    dPv   <- Pd*Ts/(nZr*s) - kv*Pv - kam*Pv # P in vegetation biomass
    dPo   <- POtterg_in*kPdig/pin + kv * Pv - (kf + kr*LQ)*Po - kta*Po - kd*fdec*Po + (1-kPdig)*kam*(pin*Pv + ps*Pv1) + (1-kPdig)*kta*(pin*Po + ps*Po1)   # P in soil organic biomass
    dPd   <- Id + Iw + POtterg_in*(1-kPdig)/pin + MaxInRN*fpr + kPdig*kam*(pin*Pv + ps*Pv1) + kPdig*kta*(pin*Po + ps*Po1) + kd*fdec*Po - (kc*Pd + Pd*Ts/(nZr*s) + Pd*LQ/(nZr*s)) 
    
    
    ######### BALANCE TERRA FIRME FOREST  ################################################
    
    dPv1   <- Pd1*Ts1/(nZr*s1) - kv * Pv1 - kam * Pv1 # P in vegetation biomass
    dPo1   <- kv * Pv1 - (kf + kr*LQ1)*Po1 - kta*Po1 - kd*fdec1*Po1 + (1-kPdig)*kam*(pin*Pv + ps*Pv1) + (1-kPdig)*kta*(pin*Po + ps*Po1)# P in soil organic biomass
    dPd1   <- Id + Iw + kPdig*kam*(pin*Pv + ps*Pv1) + kPdig*kta*(pin*Po + ps*Po1) + kd*fdec1*Po1 - (kc*Pd1 + Pd1*Ts1/(nZr*s1) + Pd1*LQ1/(nZr*s1))
    
    
    ######### TOTAL SYSTEM BALANCE  ######################################################
    
    dPr    <- - Pr + MaxInAndR*fpr*pin + Id + Iw - ps*Pd1*LQ1/(nZr*s1) - pin*Pd*LQ/(nZr*s) - ps*(kf+kr*LQ1)*Po1 - pin*(kf + kr*LQ)*Po - ps*kc*Pd1 - pin*kc*Pd
    
    return(list(c(dPv, dPo, dPd, dPv1, dPo1, dPd1, dPr)))
  })
}
### END P MODEL ######################################################################

### DEFINE PARAMETERS ################################################################

Pars <- c(temp = 25, #(*from Porder2007 *) (1)
          s = 0.7, #, (*soil moisture assuming (2)
          s1= 0.6, # 0.9  (3 soil moisture TFF) (3)
          U = 0.005757, # (*uplift rates m/kyr *); (4)
          PCon = 75, # (*concentration of P in primary mineral from  mol/m3 Porder2007*);(5)
          c = 3, #(*exponent of ruonoff/leakage function*)(6)
          kr = 0.002, #(*organic matter runoff/leakage  rate for losses*) (7)
          kl = 0.01*10, #(*runoff/leakage rate at saturation, assuming 10 leaching events per year*)(8)
          kcl = 0.7, # (*clay formationrate*)(9)
          nZr = 0.4, # (*porosity=0.4,rooting depth Zr=100 cm*) (10)
          eta = 0.005*365, #(*max transpiration rate per square meter (option2) *)(11)
          kd  = 0.00051*365, #(*decomposition rate estimated from Dodorico's nitrogen dec rate*)(12)
          kv  = 0.00055*365, #(*litterfall rate*)  (13)
          ke  = 0.00001, #(*erosion rate ke*);(14)
          Id  = 0.0004817, # Saharan dust deposition from Mahowald et al., 2005 (15)
          kc  = 0.00001, # occlution rate (16)
          eps = 1, # (17)
          kf  = 0.0001,#, # SOM removal rate (18)  
          ps  = 0.70, # % of terra firme forest (19)
          pin = 0.30, # % of flooded forest.(20)
          WeAndes= 1895.846, # moles P/km2/yr (21)
          Pmol = 30.9737622, # g P per mol (22)
          fpr = 0.8, # fraction of the maximual P transport from river to flooded(23)
          koh = 0.0, # (0-12) otter-human (oh) P River to TFF individual/km2 -1.2 observed in Guyanas(Duplex)(24)
          kam = 0.0, # (0-0.2) ant-monkey (am) hervivory and redistribution (25)
          kta = 0.0, # (0-0.2)termite-atmospheric redistribution of biomass. (26)
          kPdig = 0.5,# animal excretion ratio to available P  (27),
          IndOtterC= 3000, # g fish per day An average adult giant otter eat about(28)
          PconFish=0.034, # using the average P content of fish biomass (% **%- Vinogradov 1953, Donaldson 1967 cited in Hall 1972)(29)
          daysyear=365)

### INITIALIZATION ##############################################################################
# set pools

yini  <- c( Pv = 0, Po =0, Pd = 0.001,Pv1 =0, Po1 = 0, Pd1 = 0.001, Pr= 0)
#    1       2         3        4        5        6          7           8
#set time
times <- seq(0, 7000, length =7001)
# set par for sensitivity analysis
#sensSM <- c(0.2,0.3,0.4,0.5,0.6)
sensSM <- c(0,0.01,0.02,0.05,0.07,0.1)
sensRD <- c(0.0,0.01,0.02,0.05,0.07,0.1,0.2,0.3)
#################################################################################################


### RUN P MODEL #################################################################################

out<-list()
steady<-array(NA, c(length(sensSM), length(sensRD) ,length(yini)+1), dimnames=c("soil moisture", "redistribution rate", "P pools"))
for (i in 1: length(sensSM)){
  for (j in 1: length(sensRD)){
    
    Pars[25]<-sensSM[i] # sensitivity to the soil moisture of the tierra firme forest s1
    Pars[26]<-sensRD[j] # sensitivity to the redistribution rate ka,  it is the same for both,  TFF and FF
    print(system.time(
      out[[i*j]] <- rk(func = Pmodel, y = yini,
                       parms = Pars, times = times, hini = 0.1, method = "euler")))
    steady[i,j,] <- tail(out[[i*j]], n=1)
  }
}
### SAVE DATA ###################

write.table(steady, file="herVsDet")
PorgTFinWET<-as.matrix(steady[,,6])
write.csv(PorgTFinWET, file="PorgTFinWET.txt", row.names=T)
save(PorgTFinWET, file="PorgTFinWET.RData")

PvegTFinWET<-as.matrix(steady[,,5])
write.csv(PvegTFinWET, file="PvegTFinWET.txt", row.names=T)
save(PvegTFinWET, file="PvegTFinWET.RData")

PdisTFinWET<-as.matrix(steady[,,7])
write.csv(PdisTFinWET, file="PdisTFinWET.txt", row.names=T)
save(PdisTFinWET, file="PdisTFinWET.RData")

PvegFFinWET<-as.matrix(steady[,,2])
write.csv(PvegFFinWET, file="PvegFFinWET.txt", row.names=T)
save(PvegFFinWET, file="PvegFFinWET.RData")

PorgFFinWET<-as.matrix(steady[,,3])
write.csv(PorgFFinWET, file="PorgFFinWET.txt", row.names=T)
save(PorgFFinWET, file="PorgFFinWET.RData")

PdisFFinWET<-as.matrix(steady[,,4])
write.csv(PdisFFinWET, file="PdisFFinWET.txt", row.names=T)
save(PdisFFinWET, file="PdisFFinWET.RData")

cut(PorgTFinWET,breaks=9)
cut(PvegTFinWET,breaks=9)



####### create a PDF with the sensitivity plots ################

gcol=color = function(x)rev(grey.colors(x))
xlabN = "Herbivores consumption [%]"
ylabN = "Detritivores consumption [%] "
xlimR = c(0,5)
ylimR = c(0,20)
steady<-as.data.frame(steady)

load(file="PorgTFinWET.RData")
load(file="PvegTFinWET.RData")

load(file="PorgTFinWET.RData")
load(file="PorgTFinWET.RData")
load(file="PorgTFinWET.RData")
load(file="PorgTFinWET.RData")

pdf("AntsVstermites_RioNegro.pdf", paper="a4")
par(mfrow=c(2,2), lwd=2, pty="m")
filled.contour(sensSM*100, sensRD*100, PorgTFinWET, color=gcol, nlevels = 7, ylim=ylimR, xlim=xlimR, main=expression(paste("P in SOM in TFF g P m"^{-2})), ylab=ylabN, xlab= xlabN)

filled.contour(sensSM*100, sensRD*100, PvegTFinWET, color=gcol, nlevels = 18, ylim=ylimR, xlim=xlimR, main=expression(paste("P in vegetation in TFF g P m"^{-2})), ylab=ylabN, xlab= xlabN )

filled.contour2(sensSM*100, sensRD*100, PvegTFinWET, color=gcol, nlevels = 17, ylim=ylimR, xlim=xlimR, main=expression(paste("P in vegetation in TFF g P m"^{-2})), ylab=ylabN, xlab= xlabN )
contour(sensSM*100, sensRD*100, PvegTFinWET, add=TRUE,labcex=1, nlevels=17)

filled.contour(sensSM*100, sensRD*100, PdisTFinWET, color=gcol, nlevels = 7, ylim=ylimR, xlim=xlimR, main=expression(paste(" plant available P in TFF g P m"^{-2})), ylab=ylabN, xlab= xlabN)

filled.contour(sensSM*100, sensRD*100, PvegFFinWET, color=gcol, nlevels = 7 , ylim=ylimR, xlim=xlimR, main=expression(paste("P in vegetationin FF g P m"^{-2})), ylab=ylabN, xlab= xlabN)

filled.contour(sensSM*100, sensRD*100, PvegFFinWET, color=gcol, nlevels = 7 , ylim=ylimR, xlim=xlimR, main=expression(paste("P in vegetationin FF g P m"^{-2})))

filled.contour(sensSM*100, sensRD*100, PorgFFinWET, color=gcol, nlevels = 20, ylim=ylimR, xlim=xlimR, main=expression(paste("P in SOM FF g P m"^{-2})),   ylab=ylabN, xlab= xlabN)

filled.contour(sensSM*100, sensRD*100, PdisFFinWET, color=gcol, nlevels = 20, ylim=ylimR, xlim=xlimR, main=expression(paste("available P in FFg P m"^{-2})), ylab=ylabN, xlab= xlabN)

filled.contour(sensSM*100, sensRD*100, PorgFFinWET*0.3 + PorgTFinWET*0.7, color=gcol, nlevels = 20, ylim=ylimR, xlim=xlimR, main=expression(paste("P in SOM at the Rio Negro Basin g P m"^{-2})), ylab=ylabN, xlab= xlabN )

filled.contour2(sensSM*100, sensRD*100, PvegFFinWET*0.3 + PvegTFinWET*0.7, color=gcol, nlevels = 15 , ylim=ylimR, xlim=xlimR, main=expression(paste("P in vegetationin at Sub-basin g P m"^{-2})), ylab=ylabN, xlab= xlabN )
contour(sensSM*100, sensRD*100, PvegFFinWET*0.3 + PvegTFinWET*0.7, add=TRUE,labcex=1, nlevels=15)

filled.contour(sensSM*100, sensRD*100, PdisFFinWET*0.3 + PdisTFinWET*0.7, color =gcol, nlevels = 10, ylim=ylimR, xlim=xlimR, main=expression(paste("P plant available at the Rio Negro Basin g P m"^{-2})), ylab=ylabN, xlab= xlabN )
dev.off()